const fs = require('fs');
const path = require('path');
const puppeteer = require('puppeteer');
const axios = require('axios');

// Đường dẫn gốc, nơi chứa các folder con như acc_1, acc_2
const baseFolder = 'E:\\zzzzzzzz';

// Hàm để tìm tất cả thư mục con chứa chrome.exe và profile
function findAllChromeAndProfiles(baseFolder) {
    const directories = fs.readdirSync(baseFolder, { withFileTypes: true })
        .filter(dirent => dirent.isDirectory()) // Chỉ lấy các thư mục
        .map(dirent => path.join(baseFolder, dirent.name)); // Ghép lại đường dẫn đầy đủ

    return directories
        .map(dir => {
            const chromePath = path.join(dir, 'App', 'Chrome-bin', 'chrome.exe');
            const profilePath = path.join(dir, 'Data', 'profile');
            if (fs.existsSync(chromePath) && fs.existsSync(profilePath)) {
                return { chromePath, profilePath }; // Trả về đường dẫn đến chrome.exe và profile nếu tìm thấy
            }
            return null;
        })
        .filter(item => item !== null);
}

// Hàm để chờ trong một khoảng thời gian
function delay(time) {
    return new Promise(function (resolve) {
        setTimeout(resolve, time);
    });
}

// Hàm gọi API
async function callApi(url, authorization, data = null) {
    try {
        const headers = {
            'accept': 'application/json, text/plain, */*',
            'authorization': authorization,
            'origin': 'https://app.notpx.app',
            'referer': 'https://app.notpx.app/',
            'sec-fetch-site': 'same-site',
            'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/129.0.0.0 Safari/537.36'
        };

        const response = await axios({
            method: data ? 'POST' : 'GET',
            url: url,
            headers: headers,
            data: data
        });

        return response.data;
    } catch (error) {
        console.error(`Lỗi khi gọi API: ${error.message}`);
        return null;
    }
}

// Hàm chạy Puppeteer và thực hiện các bước
async function runForEachAccount(chromePath, profilePath) {
    try {
        // Khởi động Puppeteer với Chrome và profile đã tìm thấy
        const browser = await puppeteer.launch({
            executablePath: chromePath, // Đường dẫn đến chrome.exe tìm được
            headless: false, // Mở trình duyệt
            args: [
                '--no-sandbox',
                '--disable-setuid-sandbox',
                '--start-maximized', // Mở trình duyệt ở chế độ toàn màn hình
                `--user-data-dir=${profilePath}` // Đường dẫn đến profile
            ],
            timeout: 60000 // Tăng thời gian chờ lên 60 giây
        });

        const [page] = await browser.pages();
        await page.setViewport({ width: 1920, height: 1080 }); // Thiết lập kích thước trang

        // Truy cập vào URL ban đầu
        await page.goto('https://web.telegram.org/k/#?tgaddr=tg%3A%2F%2Fresolve%3Fdomain%3Dnotpixel%26appname%3Dapp%26startapp%3Df617272256');

        // Chờ một thời gian để trang tải hoàn chỉnh
        await delay(5000);

        // Kiểm tra xem nút "Launch" có tồn tại không
        const launchButtonSelector = '.popup-button.btn.primary.rp';
        const isLaunchButtonVisible = await page.$(launchButtonSelector);

        if (isLaunchButtonVisible) {
            console.log('Nút "Launch" đã được tìm thấy, tiến hành nhấn nút.');
            await page.click(launchButtonSelector);

            // Đợi iframe tải sau khi nhấn nút
            await delay(5000); // Điều chỉnh thời gian chờ nếu cần

            // Tìm iframe và chuyển sang nội dung bên trong iframe
            const iframeElement = await page.$('iframe');
            const iframe = await iframeElement.contentFrame();

            // Lấy URL của iframe
            const iframeUrl = await iframe.url();
            console.log('Iframe URL:', iframeUrl);

            // Bóc giá trị tgWebAppData từ URL iframe
            const tgWebAppDataMatch = iframeUrl.match(/tgWebAppData=([^&]*)/);
            if (tgWebAppDataMatch) {
                const tgWebAppData = tgWebAppDataMatch[1];
                const decodedData = decodeURIComponent(tgWebAppData);
                console.log('Decoded tgWebAppData:', decodedData);

                // Lấy authorization từ tgWebAppData
                const authorization = `initData ${decodedData}`;

                // Gọi API "mining/status"
                const miningStatusUrl = 'https://notpx.app/api/v1/mining/status';
                const miningStatus = await callApi(miningStatusUrl, authorization);

                if (miningStatus) {
                    console.log('Mining status:', miningStatus);

                    // Lấy số charges từ kết quả trả về
                    const charges = miningStatus.charges;
                    console.log(`Số charges: ${charges}`);

                    // Gọi API "repaint/start" n lần theo số charges
                    const repaintUrl = 'https://notpx.app/api/v1/repaint/start';
                    let pixelId = 748799; // Bắt đầu từ pixelId gốc

                    for (let i = 0; i < charges; i++) {
                        const repaintData = {
                            pixelId: pixelId + i,
                            newColor: '#BE0039'
                        };
                        await callApi(repaintUrl, authorization, repaintData);
                        console.log(`Đã gửi request repaint với pixelId: ${pixelId + i}`);

                        // Thêm khoảng cách 2 giây giữa mỗi request repaint
                        await delay(2000); // Chờ 2 giây (2000 ms) giữa mỗi request
                    }
                    await delay(2000);
                    // Gọi API "mining/claim" sau khi hoàn thành các request repaint
                    const claimUrl = 'https://notpx.app/api/v1/mining/claim';
                    const claimResult = await callApi(claimUrl, authorization);

                    console.log('Claim result:', claimResult);
                } else {
                    console.log('Không lấy được mining status.');
                }
            } else {
                console.log('Không tìm thấy tgWebAppData trong URL iframe.');
            }
        } else {
            console.log('Không tìm thấy nút "Launch".');
        }

        // Đóng trình duyệt sau khi hoàn tất
        await browser.close();
    } catch (error) {
        console.error(`Lỗi: ${error.message}`);
    }
}

// Hàm chính để lặp lại quá trình chạy và tạm dừng 30 phút
async function main() {
    while (true) {
        console.log('Bắt đầu chạy quy trình...');

        // Tìm tất cả các Chrome và profile trong thư mục gốc
        const accounts = findAllChromeAndProfiles(baseFolder);

        // Lặp qua tất cả các tài khoản (thư mục con)
        for (const account of accounts) {
            const { chromePath, profilePath } = account;
            console.log(`Chạy cho tài khoản với Chrome tại ${chromePath} và profile tại ${profilePath}`);
            await runForEachAccount(chromePath, profilePath);
        }

        // Dừng 30 phút sau khi hoàn tất
        console.log('Hoàn tất, dừng lại 30 phút...');
        await delay(30 * 60 * 1000); // Dừng 30 phút (30 * 60 * 1000 ms)
    }
}

// Chạy chương trình chính
main();
